
BEGIN{
  my $dir=$0;
  $dir=~s|[^/]*$|../perllib|;
  push(@INC,$dir);
}  


use PointSet;
use FacetList;
use Chirotope;
use FilterUtil qw(run_command write_file read_file);
use FilterConfig;

use Getopt::Std;

my $warncount=0;
my $errcount=0;

sub usage{
  print STDERR "usage:\n";
  print STDERR "$0 -t topcom|ine|raw|poly [-f][-i] file";
  exit(1)
}

getopts('efqi:t:m:',\%opt);

$infile=$opt{i} || shift;

$max_results=$opt{m} || 0;

$type=$opt{t} || 'om';

usage() if (!defined($infile));


if ( $type eq 'om'){
  @input=read_file($infile);
  $F=FacetList::from_mpc(@input);
  
  $input=join('',@input);

  if ($opt{m}>=1){
    $input .= "options { max_results=$opt{m} }";
  }
} else {

  $S=PointSet::input(type=>$opt{t},
		     file=>$infile);

  # optionally remove redundancy.

  $S=$S->extremes() if ($opt{e});
  $F=$S->facets();

  my @options;
  push(@options,max_results=>$max_results) if ($max_results);

  $input=$F->mpc( @options);
}
  print STDERR $input;

@output=run_command("$SRC_DIR/nuoms",$input);

if (scalar(@output)==0){
  print STDERR "ERROR: no output from O.M. search\n";
  $errcount++;
}

if (scalar(@output)==1 && $output[0] =~ m|^\#\s*no\s*solutions\s*found$|){
  print STDERR "WARNING: no solutions found. Interior points present?\n";
  $warncount++;
} elsif ($errcount==0){
  $errcount+=$F->verify_output(@output);
}
print STDERR "\n";
my $stem= $infile;
$stem =~ s/\.\w+//;

$errcount+=write_file("$stem.out",$opt{f},@output);

if ($type ne 'om'){
  $errcount+=write_file("$stem.om",$opt{f},$input);
}



print STDERR "SUMMARY: $errcount total errors, $warncount total warnings\n";


