/**
 * @file   genpath.c
 * @author David Bremner
 * @date   Thu Apr 18 12:21:38 2002
 * 
 * @brief  Generate combinatorially distinct non-revisiting paths.
 * 
 * 
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "oms_error.h"
#include "oms_types.h"
#include "cpath.h"
#include "permute.h"

static void usage(){
  printf("usage: genpath [-o outfile ] [-s ] -d dim -k steps\n");
  exit(1);
}
int main(argc, argv)
     int argc;
     char **argv;
{

  int c;
  int d=3,k=3;
  bool symmetric=0,permute=0;

  int *pi=NULL;

  while((c=getopt(argc,argv,"o:d:k:sp"))!=-1){
    switch(c){
    case 'p':
      permute=1;
      break;
    case 's':
      symmetric=1;
      break;
    case 'd':
      d=atoi(optarg);
      break;
    case 'k':
      k=atoi(optarg);
      break;
    case 'o':
      freopen(optarg,"w",stdout);
      oms_assert(stdout);
      break;
    default:
      usage();
      exit(1);
    }
  }
  
  cpath_init(d,k);

  if (permute){
    pi=calloc(d+1,sizeof(int));
  }

  do {
    cpath_next();

    if (!symmetric || cpath_check_symmetric()){ 
      if (permute){
	first_perm(pi,d);
	do {
	  cpath_output(pi);
	} while(next_perm(pi));
      }	else {
	cpath_output(NULL);
      }
    }

    cpath_remove();
  } while (cpath_more());

}
