/**
 * @file   nu_matroid.c
 * @author David Bremner <bremner@unb.ca>
 * @date   Mon Dec 30 18:21:28 2002
 * 
 * @brief matroid equations for non-uniform (the only currently
 * implemented)  case
 * 
 */


#include "basis_equation.h"
#include "combinat.h"
#include "oms_hook.h"
#include "nalloc.h"
#include "nodedef.h"
#include "globaldef.h"
#include "nu_sign_set.h"
#include "bitmask.h"
#include "debug.h"

/* note, row % rank could be used instead of all of this stupidity 
   with element 0 */
bool me_traverse(array_index_t bindex,
		 na_node_p local, 
		 na_node_p global,
		 nu_sign_set_p forced){

  sarray_p chi=na_access(local,SLOT_chirotope);
  be_block_p BE=na_access(global,SLOT_matroid_eq);
  sarray_p zero_mask=na_access(local,SLOT_matroid_zero_mask);
  sarray_p nonzero_mask=na_access(local,SLOT_matroid_nonzero_mask);

  nu_sign_t chi_bindex=sa_read(chi,bindex);


  array_index_t elem=BE->last[bindex];   

  if (!BE->initialized)
    ERR_RETURN(TRUE);


  err_assert( elem >= -1 && elem < BE->rows*BE->cols );
  while ( elem>=0  ){
    array_index_t row,col,row_start;
    bitmask_t zmask,nzmask;
    nu_sign_t sigma0;
    bool row_active, has_neighbours;

    row=be_row(BE,elem);
    col=be_col(BE,elem);
    row_start=be_row_start(BE,elem);
    
    sigma0=sa_read(chi,be_bindex_raw(BE,row_start));

    nzmask=be_update_mask(nonzero_mask,chi_bindex,row,col,SIGN_NONZERO);
    zmask=be_update_mask(zero_mask,chi_bindex,row,col,SIGN_ZERO);

    row_active=( (nzmask&BIT0_MASK) != 0);
    has_neighbours=( (nzmask&(~BIT0_MASK)) != 0);
    
    err_debug(DB_MATROID,(HERE,"row=%d, nzmask=%x zmask=%x",
			  row,nzmask,zmask));


    if (row_active){
      if ( (nzmask|zmask)==BE->ones_mask){
	/* all bits set */
	
	/* sign check is for rows with no possible neighbours,
	   i.e. root basis*/

	if ((!has_neighbours) && be_sign(BE,row,1)>0)
	  ERR_RETURN(FALSE);
	
      } else {   
	if (!has_neighbours){
	  /* row active, but no neighbours set */
	
	  bitmask_t missing_bit= (~(nzmask|zmask))&BE->ones_mask&(~BIT0_MASK);
	  if (missing_bit && ZERO_OR_POWER_OF_TWO(missing_bit)){
	    array_index_t basis,array_index_t;
	  
	    err_debug(DB_MATROID,(HERE, " nzm %x zm %x mb %x ",
				  nzmask,zmask,missing_bit));
	  
	    array_index_t=be_row_start(BE,elem)+LOG2(missing_bit);
	    basis=be_bindex_raw(BE,array_index_t);
	
	    err_debug(DB_MATROID|DB_FORCE,
		      (HERE, "row %d forcing %d (%x) to nonzero",
		       be_row(BE,elem), basis,missing_bit));

	    if (!nss_add(forced,basis,SIGN_NONZERO)) ERR_RETURN(FALSE);
	  }
	}
      }
    }
    elem=BE->eqn[elem].prev;  
  } /* while (elem>0) */

  ERR_RETURN(TRUE);

  

}
