/**
 * @file   z_buf.c
 * @author David Bremner <bremner@unb.ca>
 * @date   Wed Apr  3 10:55:53 2002
 * 
 * @brief  manage inbuffer and outbuffer for zram
 * 
 * Currently only z_md_mpi is converted to use this.
 */

#include <stddef.h>
#include <stdlib.h>

#ifdef HAVE_CONFIG_H
#include "zram_config.h"
#endif

#ifdef USE_DMALLOC
#include <dmalloc.h>
#endif

#include "z_md.h"

struct buffer *inbuffer, *outbuffer; 

long z_bufunits;
long z_bufsize;

static int z_buf_inited=0;

/** 
 * allocate input/output buffers for zram
 * 
 * @param units how many bytes to allocate
 * 
 */
void z_buf_alloc(size_t bytes){


  z_bufunits=bytes/sizeof(unitT)+1;
  z_bufsize=z_bufunits*sizeof(unitT);
  inbuffer = (struct buffer *)
    malloc(sizeof(struct buffer_header) + z_bufunits * sizeof(unitT));
  assert(inbuffer);
  outbuffer = (struct buffer *)
    malloc(sizeof(struct buffer_header) + z_bufunits * sizeof(unitT));
  assert(outbuffer);
  z_buf_inited=1;

}

/** 
 * resize input/output buffers for zram
 * 
 * @param units how many bytes to allocate
 * 
 */

void z_buf_realloc(size_t bytes){
  z_bufunits=bytes/sizeof(unitT)+1;
  z_bufsize=z_bufunits*sizeof(unitT);

  assert(z_buf_inited);

  inbuffer = (struct buffer *)
    realloc(inbuffer,sizeof(struct buffer_header) + z_bufsize);
  assert(inbuffer);
  outbuffer = (struct buffer *)
    realloc(outbuffer,sizeof(struct buffer_header) + z_bufsize);
  assert(outbuffer);
}

/** 
 * Resize buffers if necessary
 * 
 * @param bytes must fit at least this many
 * 
 */
void z_buf_maybe_realloc(size_t bytes){
  assert(z_buf_inited);
  if (bytes>z_bufsize){
    z_buf_realloc(bytes);
  }
}

