/* z_md_mpi.c (MPI machine dependent functions)
              for homogeneous networks only

 * $Source$
 * $Revision: 3145 $
 * $Date: 2004-08-25 18:01:45 -0300 (Wed, 25 Aug 2004) $
 * $Log$
 * Revision 1.12  2004/08/25 21:01:45  bremner
 * libutil configuration
 * zram without mpi
 *
 * Revision 1.11  2002/07/22 15:43:46  bremner
 * *** empty log message ***
 *
 * Revision 1.10  2002/07/18 15:21:22  bremner
 * *** empty log message ***
 *
 * Revision 1.9  2002/04/08 07:54:27  bremner
 * fixing some memory bugs.  Also filled in some static initialization values.
 *
 * Revision 1.8  2002/04/05 15:31:41  bremner
 * Fixed bad assumption about input buffer size.
 *
 * Revision 1.7  2002/04/03 11:50:53  bremner
 * tracking ugly memory leaks
 *
 * Revision 1.6  2002/04/02 15:28:39  bremner
 * Fixed bug with forgetting to forget pos_mask and neg_mask.
 *
 * Revision 1.5  2001/08/27 02:44:48  bremner
 * snapshot
 *
 * Revision 1.4  2001/08/09 20:51:46  bremner
 * malloc fun continues.  Leakage mostly staunched for short1.om
 *
 * Revision 1.3  2001/08/08 21:07:41  bremner
 * beginnings of a configuration system. Mostly I am bored with creating
 * new revisions of Makefile
 *
 * Revision 1.2  2000/11/15 16:14:52  bremner
 *
 * committing patched version of zram source
 *
 * Revision 1.3  1996/06/03  07:19:05  marzetta
 * Removed unused variables
 *
 * Revision 1.2  1996/04/09  13:21:42  marzetta
 * Cenju-3
 *
 * Revision 1.1  1995/08/22  12:43:12  marzetta
 * Initial revision
 *
 */

#include <mpi.h>
#include <stdlib.h>		/* malloc() */
#include <sys/types.h>		/* for kill */
#include <signal.h>		/* for kill */
#include "z_md.h"
#include "z_par_intern.h"
#include <stdio.h>		/* debugging */

#ifdef HAVE_CONFIG_H
#include "zram_config.h"
#endif

#ifdef USE_DMALLOC
#include <dmalloc.h>
#endif

const boolean z_md_doesTD = 0;

#ifdef cenju3
#define MAXSIZE 100000
#else
#define MAXSIZE 2147483647
#endif
				/* maximum message size (sizeof units) */
				/* may be smaller than buffer size */
				/* used only if undefined(IMM_SEND) */

#define Mega 1024*1024

#ifndef ZRAM_MEMSIZE
#define ZRAM_MEMSIZE 128
#endif

long z_memsize = ZRAM_MEMSIZE * Mega;	/* nobi / tobi: 128 MByte real memory */
long z_memavail = ZRAM_MEMSIZE * Mega;	/* nobi / tobi: 128 MByte real memory */

#define mpi(f) {z_signal_Block(); mpi_err = f; assert(!mpi_err); z_signal_UnBlock();}
static int mpi_err;

/* Message Types ------------------------------------------------------

   0: MT_GEN, general
   1: MT_START_JOB, start a parallel job
   2: MT_COLLECT_RESULT
   3: MT_CONT, continuation of another message
   */

#define MT_GEN 0
#define MT_START_JOB 1
#define MT_COLLECT_RESULT 2
#define MT_CONT 3

/* Send / Receive ------------------------------------------------ */

static boolean sending;

#define BUFSIZE 1000000
extern long z_bufunits;
  
#define BCAST_DEGREE 128

static MPI_Request recv_req;
static MPI_Status recv_stat;

#ifdef IMM_SEND

/* This part is compiled if the MPI implementation supports Isend and
   Testall. (MPICH on Alpha workstation) */

static MPI_Request send_req[BCAST_DEGREE];
static MPI_Status send_stat[BCAST_DEGREE];
static int nsend_req;

boolean z_md_TestSend(void)
{
  /* (PE only) Is it possible to overwrite outbuffer? */
  int complete;
#ifdef TRACE1
  fprintf(stderr, "[%2d]CanSend %ld\n", z_myproc, z_bufunits);
  fflush(stdout);
#endif  
  assert(sending);
  mpi(MPI_Testall(nsend_req, &send_req[0], &complete, &send_stat[0]));
  sending = ! complete;
#ifdef TRACE1
  printf("[%2d]Sending %scomplete %d %d\n", z_myproc,
	 complete ? "" : "not ", complete, sending);
#endif  
  return complete;
} /* z_md_TestSend */

void z_md_Isend(struct buffer *buffer, long size, int dest,
		const char *comment)
{
  /* Send outbuffer to dest (immediate send) */
#ifdef TRACE
  printf("[%2d]Send %s\n", z_myproc, comment); fflush(stdout);
#endif  
  assert(! sending);
  buffer->header.treeRoot = -1;
  mpi(MPI_Isend(buffer, size, MPI_BYTE, dest,
		MT_GEN, MPI_COMM_WORLD, &send_req[0]));
  sending = 1;
  nsend_req = 1;
  /*printf("xx %lx\n", send_req[0]);*/
} /* z_md_Isend */

void z_md_Ibroadcast(struct buffer *buffer, long size, const char *comment)
{
  /* Broadcast outbuffer */
  int i;
  assert(! sending);
  buffer->header.treeRoot = z_myproc;
#ifdef TRACE
  printf("[%2d]Broadcast\n", z_myproc);
#endif  
  assert(z_numprocs <= BCAST_DEGREE + 1);
  for (i=0; i<z_numprocs-1; i++)
      mpi(MPI_Isend(buffer, size, MPI_BYTE, (z_myproc+1+i) % z_numprocs,
		    MT_GEN, MPI_COMM_WORLD, &send_req[i]));
  sending = 1;
  nsend_req = z_numprocs-1;
} /* z_md_Ibroadcast */

#else /* ! IMM_SEND */

/* This part is compiled if the MPI implementation does not support Isend and
   Testall. (Paragon, Cenju-3) */

boolean z_md_TestSend(void)
{
  /* (PE only) Is it possible to overwrite outbuffer? (no immediate send) */
#ifdef TRACE
  fprintf(stderr, "[%2d]CanSend %ld\n", z_myproc, z_bufunits);
#endif  
  /*for (i=0; i< BCAST_DEGREE; i++) printf("%lx ", send_req[i]);*/
  /*printf("\n");*/
#ifdef TRACE
  fflush(stdout);
#endif  
  assert(sending);
  sending = 0;
  return 1;
} /* z_md_TestSend */

void z_md_Isend(struct buffer *buffer, long size, int dest,
		const char *comment)
{
  /* Send outbuffer to dest (no immediate send) */
  long sizeNow, sent;
  
#ifdef TRACE
  printf("[%2d]Send %s\n", z_myproc, comment); fflush(stdout);
#endif  
  assert(! sending);
  buffer->header.treeRoot = -1;
  buffer->header.len = size;
  sent = 0;
  while (size > 0) {
      /*if (sent) printf("continuation send\n");*/
      sizeNow = (size > MAXSIZE) ? MAXSIZE : size;
      mpi(MPI_Send((char *)buffer + sent, sizeNow, MPI_BYTE, dest,
		   MT_GEN, MPI_COMM_WORLD));
      size -= sizeNow;
      sent += sizeNow;
  }
  sending = 1;
} /* z_md_Isend */

void z_md_Ibroadcast(struct buffer *buffer, long size, const char *comment)
{
  /* Broadcast outbuffer (no immediate send) */
  int i;
  long size1, sizeNow, sent;
  assert(! sending);
  buffer->header.treeRoot = z_myproc;
  buffer->header.len = size;
#ifdef TRACE
  printf("[%2d]Broadcast\n", z_myproc);
#endif  
  assert(z_numprocs <= BCAST_DEGREE + 1);
  for (i=0; i<BCAST_DEGREE; i++)
      if (z_numprocs >= 2+i) {
	  size1 = size;
	  sent = 0;
	  while (size1) {
	      sizeNow = (size1 > MAXSIZE) ? MAXSIZE : size1;
	      /*if (sent) printf("continuation broadcast\n");*/
	      mpi(MPI_Send((char *)buffer + sent, sizeNow, MPI_BYTE,
			   (z_myproc+1+i) % z_numprocs,
			   MT_GEN, MPI_COMM_WORLD));
	      size1 -= sizeNow;
	      sent += sizeNow;
	  }
      }
  sending = 1;
} /* z_md_Ibroadcast */

#endif /* IMM_SEND */

void ReceiveRest(int source)
{
    /* Test if the received message has been segmented. If so, receive the
       rest. */
    unsigned long received;
    MPI_Status status;
    
    received = MAXSIZE;
    while (received < inbuffer->header.len) {
	/*printf("continuation receive\n");*/
	mpi(MPI_Recv((char *)inbuffer+received,
		     z_bufsize + sizeof(struct buffer_header),
		     MPI_BYTE,
		     source,
		     MPI_ANY_TAG,
		     MPI_COMM_WORLD,
		     &status));
	received += MAXSIZE;
    }
} /* ReceiveRest */
    
#if 0
static void TestBroadcast(void)	/* not used in this version */
{
  int mypos, left;
  if (inbuffer->header.treeRoot != -1) {
    mypos = (z_myproc - inbuffer->header.treeRoot + z_numprocs) % z_numprocs;
    assert(mypos);
    left = 2 * mypos + 1;
    if (left < z_numprocs) ;
  }
} /* TestBroadcast */
#endif

void z_md_Irecv(void)
{
  mpi(MPI_Irecv(inbuffer,
		z_bufsize + sizeof(struct buffer_header),
		MPI_BYTE,
		MPI_ANY_SOURCE, MT_GEN, MPI_COMM_WORLD, &recv_req));
} /* z_md_Irecv */

boolean z_md_TestRecv(void)
{
  int complete;
  mpi(MPI_Test(&recv_req, &complete, &recv_stat));
  /*if (complete) TestBroadcast();*/
  if (complete) ReceiveRest(recv_stat.MPI_SOURCE);
  return complete;
} /* z_md_TestRecv */

boolean z_md_WaitRecv(void)
/* The processing element is idle.
   There is a receive pending.
   Wait for the receive operation started by z_StartReceive to complete.
   Return true if/when all processing elements are idle.
   Return false if/when a message is received.
   Post: There is no receive pending.
   */
{
#ifdef TRACE
  printf("[%2d]Wait\n", z_myproc);
#endif  
  z_signal_Block();
  mpi(MPI_Wait(&recv_req, &recv_stat));
  z_signal_UnBlock();
#ifdef TRACE
  printf("[%2d]Wait end\n", z_myproc);
#endif  
  ReceiveRest(recv_stat.MPI_SOURCE);
  /*TestBroadcast();*/
  return 0;
} /* z_md_WaitRecv */


/* Starting jobs ---------------------------------------------------- */

void z_md_StartParallel(void)
{
  /* Broadcast outbuffer and initiate the parallel computation. */
  int count, sizeNow, sent;
  count = outpos * sizeof(unitT) + sizeof(struct buffer_header);
  mpi(MPI_Bcast(&count, 1, MPI_INT, 0, MPI_COMM_WORLD));
  sent = 0;
  while (sent < count) {
      /*if (sent) printf("cont broadcast\n");*/
      sizeNow = count - sent > MAXSIZE ? MAXSIZE : count - sent;
      mpi(MPI_Bcast((char*)outbuffer+sent, sizeNow, MPI_BYTE,
		    0, MPI_COMM_WORLD));
      sent += sizeNow;
  }
} /* z_md_StartParallel */

/** \todo see if anywere else in this file has nasty assumptions 
    about buffer size */

void z_md_RecBuffer(void)
{
  /* Called from z_pe_loop */
  int count, sizeNow, received;
  mpi(MPI_Bcast(&count, 1, MPI_INT, 0, MPI_COMM_WORLD));
  z_buf_maybe_realloc(count);
  received = 0;
  while (received < count) {
      sizeNow = count - received > MAXSIZE ? MAXSIZE : count - received;
      mpi(MPI_Bcast((char*)inbuffer+received, sizeNow, MPI_BYTE,
		    0, MPI_COMM_WORLD));
      received += sizeNow;
  }
} /* z_md_RecBuffer */


/* Ending jobs ------------------------------------------------------- */

void z_md_Wait(z_ResultProc resultProc)
{
  int numcollected, sender;
  MPI_Status status;

  assert(z_myproc == 0);
  if (resultProc) {
    numcollected = 1;

    while (numcollected < z_numprocs) {
      mpi(MPI_Recv(inbuffer,
		   z_bufsize + sizeof(struct buffer_header),
		   MPI_BYTE,
		   MPI_ANY_SOURCE,
		   MT_COLLECT_RESULT,
		   MPI_COMM_WORLD,
		   &status));
      sender = status.MPI_SOURCE;
      ReceiveRest(sender);
      inpos = 0;
      /*printf("Got result from PE %d\n", sender);*/
      resultProc(sender);
      numcollected ++;
    }
  }
} /* z_md_Wait */

void z_md_SendToFE(boolean collect)
/* (PE only) Called by z_pe_switch if !z_frontend.
   Send the result of the computation to the front end which receives
   it in z_md_Wait.
   */
{
    long sizeNow, sent, size;
    
    if (collect) {		/* send result back to front end */
	size = outpos * sizeof(unitT) + sizeof(struct buffer_header);
	outbuffer->header.len = size;
	sent = 0;
	while (size > 0) {
	    /*if (sent) printf("continuation sendtofe\n");*/
	    sizeNow = (size > MAXSIZE) ? MAXSIZE : size;
	    mpi(MPI_Send((char *)outbuffer + sent, /* MPI_Ssend */
			  sizeNow,
			  MPI_BYTE,
			  0, /* destination: front end */
			  MT_COLLECT_RESULT,
			  MPI_COMM_WORLD));
	    size -= sizeNow;
	    sent += sizeNow;
	}
    }
} /* z_md_SendToFE */

/* Assert ---------------------------------------------------------- */

void z_md_abort(void)
{
#if 1
  abort();
#else
  (void) MPI_Abort(MPI_COMM_WORLD, 1); /* no core dump */
#endif
} /* z_md_abort */

/* Initialization --------------------------------------------------- */

void z_md_Init(int *argc, char ***argv)
/* Initialize this module. Called by main(). */
{
  int size, rank;
  
  z_buf_alloc(BUFSIZE);

  mpi(MPI_Init(argc, argv));
  mpi(MPI_Comm_size(MPI_COMM_WORLD, &size));
  mpi(MPI_Comm_rank(MPI_COMM_WORLD, &rank));
  z_par_Init(rank, size, rank == 0);
} /* z_md_Init */

void z_md_Fin(void)
{
  mpi(MPI_Finalize());
} /* z_md_Fin */
