/* z_time.c

   Ambros Marzetta

 * $Source$
 * $Revision: 2732 $
 * $Date: 2000-11-15 12:14:52 -0400 (Wed, 15 Nov 2000) $
 * $Log$
 * Revision 1.2  2000/11/15 16:14:52  bremner
 * committing patched version of zram source
 *
 * Revision 1.6  1997/07/18  14:23:46  marzetta
 * Paragon/1 processor is now correct.
 *
 * Revision 1.5  1997/04/26  15:35:13  marzetta
 * Portable to more systems.
 *
 * Revision 1.4  1996/05/29  17:30:52  marzetta
 * Removed definition of gettimeofday for solaris
 *
 * Revision 1.3  1996/05/15  16:44:18  marzetta
 * included z_base.h for macintosh version
 *
 * Revision 1.2  1996/04/09  13:19:00  marzetta
 * Cenju-3
 *
 * Revision 1.1  1995/08/22  12:39:58  marzetta
 * Initial revision
 *
 */

#include "z_time.h"
#include "z_base.h"

#ifdef macintosh
#include <time.h>
#elif defined(A7)
#include "c_pal.h"
#else /* not A7 */
#include <sys/time.h>		/* all Unix systems */
#include <sys/resource.h>	/* struct rusage */
#include <stdio.h>		/* perror */
#ifdef __osf__			/* OSF:  getclock, getrusage */
#include <sys/timers.h>
int getclock(int clock_type, struct timespec *tp);
#elif defined(cenju3)
#include <mpi.h>
#elif defined(sunbsd) /* SunOS 4 */
int gettimeofday(struct timeval *, struct timezone *);
#elif defined(__HP__) || defined (__sun)
#include <sys/times.h>
#else /* Unix, but not OSF and not SunOS 4*/
int getrusage(int who, struct rusage *rusage); /* BSD */
#endif /* not __osf__ */
#endif /* not A7 */

/* elapsed time:
   gettimeofday		BSD, (IRIX), HP/UX, NeXT, (Solaris), not OSF/1
   getclock		OSF/1, (HP/UX)
   MPI_Wtime		MPI, Cenju-3
   time			IRIX, HP/UX (resolution = 1s)
   
   process time:
   getrusage		BSD, OSF/1, SunOS4, NeXT, (IRIX), (Solaris)
   times		HP/UX, IRIX, Solaris, (OSF/1), (NeXT)
   clock		(IRIX), Macintosh (wraps around)
   */

double z_Wtime()
/* Return the number of seconds elapsed since some starting point
   (wall clock time). Must be saved as double, not as float.
 */
{
#if defined(A7)
  return 1.0 * (65536 - os_rtc_readt()) / 1000;
#elif defined(macintosh)
  return (1.0 * clock() / CLOCKS_PER_SEC);
#elif defined(__osf__)
  struct timespec tspec;
  
  (void) getclock(TIMEOFDAY, &tspec);
  /*printf("    time: %9ld %9ld\n", (long)tspec.tv_sec, (long)tspec.tv_nsec);*/
  return ((double)1.0 * tspec.tv_sec + 0.000000001 * tspec.tv_nsec);
#elif defined(cenju3)
  return MPI_Wtime();
#else /* Unix, but not OSF */
  struct timeval tp;

  gettimeofday(&tp, 0);
  /*printf("    time: %9ld %6ld\n", (long)tp.tv_sec, (long)tp.tv_usec);*/
  return ( (double)1.0 * tp.tv_sec + 0.000001 * tp.tv_usec);
#endif
} /* z_Wtime */

double z_Ptime(void)
/* Return the number of cpu seconds used by the current process.
   Must be saved as double, not as float.
 */
{
#ifdef A7
  return 1.0 * (65536 - os_rtc_readt()) / 1000;
#elif defined(macintosh)
  return z_Wtime();
#elif defined(sunbsd) || defined(__osf__) || defined(__sgi) || defined (__NeXT__) || defined(__PARAGON__)
  struct rusage rusage;

  if (getrusage(RUSAGE_SELF, &rusage)) perror("getrusage");
  return ((double)1.0 * rusage.ru_utime.tv_sec +
	  0.000001 * rusage.ru_utime.tv_usec +
	  (double)1.0 * rusage.ru_stime.tv_sec +
	  0.000001 * rusage.ru_stime.tv_usec);
#elif defined (__HP__) || defined (__sun)
  struct tms buffer;
  
  if (times(&buffer) == -1) perror("times");
  return ((double)buffer.tms_utime + buffer.tms_stime) / CLK_TCK;
#elif defined(cenju3)
  return MPI_Wtime();
#else
  return 0.0;
#endif
} /* z_Ptime */
